 ;======================================================================
; Create a fake ACS WFC .flt image with simulated GC in random positions
; using king_addACS.pro to simulate GC and the Anderson psf library to create the appropriate psf
; Requires the dithered ACS image and the auxiliaries coords files prodced by drizzle
; to convert coords to astrometry
; Written on 1/6/04 by M.Paolillo; edited on 15/11/10
;
; ** NOTE: X,Y pos within pixel still manually fixed if simulating, derived from Ra,Dec if reading
; ** NOTE: Filter is read from input image but counts/flux conversion
;          must be changed manually in kingmod_addacs.pro
; ** NOTE: det1 is upper one, det2 is lower one, but the fits extensions in the final 
;   	    image follow the opposite rule
; ** NOTE: Anderson PSF ok. Instead Tinytim PSF is still created at 1:1 resolution and is centered on pixel - it's use is obsolete
;======================================================================


resolve_routine, "kingmod_addacs", /compile_full_file	;Compile whole king model procedure (must be in same directory)

; --- Initial parameters ---

xdimgc=100.  ; x size of simulated GC image
ydimgc=100.  ; y size of simulated GC image

xdim=4096.   ; Total detector size - both chips
ydim=4096.


; --- Read original ACS image header
refimg=''
read, refimg, prompt='Reference ACS flt image? '

primary_head=headfits(refimg, exten=0)
det1_head=headfits(refimg, exten=4) ; det. 1 (upper, fits extension [sci,2])
det2_head=headfits(refimg, exten=1) ; det. 2 (lower, fits extension [sci,1])


; -- Extract info from primary header --
exptime = sxpar(primary_head, 'EXPTIME')    ; extract exposure time from the primary header
print, 'Exposure time = ',exptime, ' s ' 

   print, '!!========================!!!!!!!!!!==============================!!'
   print, '  WARNING! The cnts/flux conversion is hardcoded in the kingmod_addacs.pro',$
    	  ' procedure. Edit it if you want to change them !'
   print, '!!========================!!!!!!!!!!==============================!!'
   
filt1 = sxpar(primary_head, 'FILTER1')	; ACS filter on wheel 1
filt2 = sxpar(primary_head, 'FILTER2')	; ACS filter on wheel 2

if ( strpos(filt1,'CLEAR') eq -1 and strpos(filt2,'CLEAR') ne -1 ) then filt = strtrim(filt1,2) $
    else if ( strpos(filt1,'CLEAR') ne -1 and strpos(filt2,'CLEAR') eq -1 ) then filt = strtrim(filt2,2) $
    else print, 'Unknown ACS filters combination:', strtrim(filt1,2), strtrim(filt2,2)

print, '-> ACS filter:', filt



; ----- Acquire further input parameters ------------
dithered_img=""
read, dithered_img, prompt='Final dithered reference ACS image (_drz_sci file)? '
dithered_head=headfits(dithered_img, exten=0) ; Read dithered header for WCS info

outimg=""   ;define string variable
read, outimg, prompt='Output simulated ACS image with GC? '
print, outimg	;debug line

; Add simulation to original image or create new image?
finimgflag=''  ; define string variable
read, finimgflag, prompt='Add simulation to original image or create new image? [a/n]'

; Simulate GC or PSF?
psfflag=''  ; define string variable
read, psfflag, prompt='Simulate GC or PSF (Note: you have to specify tentative GC parameters anyway)? [gc/psf]'

jump1:
simflag=''  ; define string variable
read, simflag, prompt='Perform simulations or read GC parameters from file? [p/r]'

if (simflag eq "p") then begin   ; create simulated GC parameters

   outlist=""  ; define string variable
   read, outlist, prompt='Output file with simulated GC parameters? '

   read, numgc, prompt='Number of GC to simulate (>1)? '

   ; Define double precision arrays
   RA=dblarr(numgc)
   Dec=dblarr(numgc)

   ; --- set king model parameters ----
   read, r0min, r0max, prompt='Core radius range in pixels [min max]? '
   read, cmin, cmax, prompt='Concentration index range [min max]? '
   read, totmag_min, totmag_max, prompt='Total V magniture range [min max]? '

   ; generate king model parameters
   r0=replicate(0.,numgc)  ; define parameter arrays
   c=replicate(0.,numgc)   ; define parameter arrays
   totmag=replicate(0.,numgc)	   ; define parameter arrays
   
   xpixcent=replicate(50.,numgc)	; X position of GC center within simulated image
   ypixcent=replicate(50.,numgc)	; Y position of GC center within simulated image
   print, "!!========================!!!!!!!!!!==============================!!"
   print, "WARNING: GC position is fixed to ("+string(xpixcent[0], format='(f5.2)')+","+$
   	string(ypixcent[0], format='(f5.2)')+") within the simulated sub-image!"
   print, "For correct convolution the GC image must be approx. centered (avoid edges)!"
   print, "!!========================!!!!!!!!!!==============================!!"


   r0=r0min+randomu(seed_r0,numgc)*(r0max-r0min)   ; generate random sequence
   c=cmin+randomu(seed_c,numgc)*(cmax-cmin)		   ; generate random sequence
   totmag=totmag_min+randomu(seed_totmag,numgc)*(totmag_max-totmag_min)    ; generate random sequence

 
   ; Use following line to manually set fixed CG parameters
   ;r0=0.1	; Core radius
   ;c=50.      ; Concentration index c=r_t/r0
   ;totmag=22.  ; total magnitude (in V band)
   
   ; Open file for GC list output
   openw, 11, outlist, width=300
   printf, 11, "---- Parameters of the simulated ACS Globular Clusters -----"
   printf, 11, "-- Output image: ", outimg
   printf, 11, "-- Input parameters: "
   printf, 11, "-- x size of simulated GC image: ", xdimgc
   printf, 11, "-- y size of simulated GC image: ", ydimgc
   printf, 11, "-- Total detector size - both chips: ", xdim, " x ", ydim
   printf, 11, "-- Exposure time = ",exptime
   printf, 11, "-- ACS filter: ", filt
   printf, 11
   printf, 11, "       N                RA              Dec              X         Y           r_0	  Conc.index   Tot.mag.      X,Y cent.in pixel         WFC chip         PSF file"

endif else if (simflag eq "r") then begin   ; read simulated GC parameters from file
   
   outlist=""  ; define string variable
   read, outlist, prompt='Input file with simulated GC parameters? '
   
   readcol, outlist, GCnum, RA, Dec, xcent, ycent, r0, c, totmag, xpixcent, ypixcent, chipnum, format='I,D,D,D,D,F,F,F,F,F,I,A', skipline=10
  
   numgc=n_elements(GCnum)  ; number of GC to simulate (>2)

   outlist2=""  ; define string variable
   read, outlist2, prompt='Output file with GC parameters for reference image? '

   openw, 11, outlist2, width=300
   printf, 11, "---- Parameters of the simulated ACS Globular Clusters -----"
   printf, 11, "-- Output image: ", outimg
   printf, 11, "-- Input parameters: "
   printf, 11, "-- x size of simulated GC image: ", xdimgc
   printf, 11, "-- y size of simulated GC image: ", ydimgc
   printf, 11, "-- Total detector size - both chips: ", xdim, " x ", ydim
   printf, 11, "-- Exposure time = ", exptime
   printf, 11, "-- ACS filter: ", filt
   printf, 11
   printf, 11, "       N                RA              Dec              X         Y           r_0	  Conc.index   Tot.mag.      X,Y cent.in pixel         WFC chip         PSF file"

endif else goto, jump1	; if wrong choice go back



weightflag=""	; define string variable
read, weightflag, prompt='Modify original weight masks to include sim.source Poisson noise (assumes ERR weighting in multidrizzling) [y/n]? '
if (weightflag eq 'y' and finimgflag eq 'n') then print, 'WARNING: masks will not be modified if creating new image!'
if (weightflag eq 'y' and finimgflag ne 'n') then begin
   fltprefix=strmid(refimg,0,strpos(refimg,'.fits'))	; extract flt filename prefix, assuming standard xxxx_flt.fits naming convention
   print, 'WARNING: assumes output image has extension .fits !'
   print, outimg
   simmask1=strmid(outimg,0,strpos(outimg,'.fits'))+'_mask1.fits'
   simmask2=strmid(outimg,0,strpos(outimg,'.fits'))+'_mask2.fits'
   spawn, 'cp '+fltprefix+'_final_mask1.fits '+simmask1  ; duplicate mask files to be modified
   spawn, 'cp '+fltprefix+'_final_mask2.fits '+simmask2
endif




; --- Start procedure ---

acs_det1_img = make_array(xdim, ydim/2, /double)	; create new ACS-like array for det. 1 (upper)
acs_det2_img = make_array(xdim, ydim/2, /double)	; create new ACS-like array for det. 2 (lower)

if (simflag eq "p") then begin   ; create simulated GC positions if 'perform' option is chosen

   ; Create random set of X,Y coordinates away from edges to add GC image excluding regions within
   ; 1/2 the GC image near borders and the interchip gap. Y coordinates are created on a 4096 scale
   ; and then assigned to the correct chip: remember that Y(det1)>Y(det2) since det1 is the upper one
   ; These will be the position of the center of the simulated GC images, not the peak of the GC itself,
   ; which is assigned by (xpixcent,ypixcent) within the simulated GC images
   
   ;xseed=111	; for test purposes fix random sequence, comment for random positions
   ;yseed=123	; for test purposes fix random sequence, comment for random positions
   xcent=fix((xdim-xdimgc)*randomu(xseed, numgc))+fix(xdimgc/2)
   ycent_det1=fix( ((float(ydim)/2.)-ydimgc) * randomu(yseed, numgc-fix(float(numgc)/2.)) ) + fix(ydimgc/2) + fix(float(ydim)/2.)
   ycent_det2=fix( ((float(ydim)/2.)-ydimgc) * randomu(yseed, fix(float(numgc)/2.)) ) + fix(ydimgc/2)
   ;print, xseed, yseed	    ; for test purposes 
   ;xcent[0]=2072   ; for test purposes fix 1st GC position to the WFC ref.point
   ;ycent_det1[0]=2248   ; for test purposes fix 1st GC position to the WFC ref.point
   ycent=[ycent_det1,ycent_det2]   ; total Y coord. array


   
   ; Convert each position in dithered image coords using the
   ; tran pyraf routine and then in RA, Dec coords for use in dithered images
   
   ; Sources on det.1 (upper), fits extension: [sci,2]
   openw, 22, 'GC_xypos.tmp', width=300	; open tmp coord file
   for iaa=0,n_elements(ycent_det1)-1 do printf, 22, xcent[iaa]+(xpixcent[iaa]-xdimgc/2.),$
    	ycent_det1[iaa]-fix(float(ydim)/2.)+(ypixcent[iaa]-ydimgc/2.) ;write x,y flt coords in tmp file
   close, 22
   spawn, 'source get_dithered_coord.csh '+string(refimg)+' '+string(dithered_img)+$
    	' 2 GC_xypos.tmp forward'
   readcol, 'get_dithered_coord.log', Xflt, Yflt, Xdth, Ydth;, skipline=7
   xyad, dithered_head, Xdth-1., Ydth-1., RA_det1, Dec_det1 ; Note! XYAD works with IDL arrays starting at [0,0] !
   
   ; Sources on det.2 (lower), fits extension: [sci,1]
   openw, 22, 'GC_xypos.tmp', width=300	; open tmp coord file
   for iaa=0,n_elements(ycent_det2)-1 do printf, 22, xcent[iaa+n_elements(ycent_det2)]+$
   	(xpixcent[iaa+n_elements(ycent_det2)]-xdimgc/2.),ycent_det2[iaa]+$
	(ypixcent[iaa+n_elements(ycent_det2)]-ydimgc/2.) ;write x,y flt coords in tmp file
   close, 22
   spawn, 'source get_dithered_coord.csh '+string(refimg)+' '+string(dithered_img)+$
    ' 1 GC_xypos.tmp forward'
   readcol, 'get_dithered_coord.log', Xflt, Yflt, Xdth, Ydth;, skipline=7   
   xyad, dithered_head, Xdth-1., Ydth-1., RA_det2, Dec_det2

   RA = [RA_det1, RA_det2]
   Dec = [Dec_det1, Dec_det2]

endif else if (simflag eq "r") then begin   ; read simulated GC positions if 'read' option is chosen

   RA_det1=RA[where(chipnum eq 1)]   ; split coordinates according to chip number
   Dec_det1=Dec[where(chipnum eq 1)]
   RA_det2=RA[where(chipnum eq 2)]
   Dec_det2=Dec[where(chipnum eq 2)]
   
   ; Non-integer pixel positions on a 4096x2048 scale are derived from astrometry

   ; Sources on det.1 (upper), fits extension: [sci,2]
   adxy, dithered_head, RA_det1, Dec_det1, Xdth, Ydth   ; Note! ADXY works in with IDL arrays starting at [0,0] !
   openw, 22, 'GC_xypos.tmp', width=300	; open tmp coord file
   for iaa=0,n_elements(Xdth)-1 do printf, 22, Xdth[iaa]+1., Ydth[iaa]+1. ;write x,y dith. coords in tmp file correcting to image coordinates (first pixel=[1,1])
   close, 22
   spawn, 'source get_dithered_coord.csh '+string(refimg)+' '+string(dithered_img)+$
    	' 2 GC_xypos.tmp backward'
   readcol, 'get_dithered_coord.log', frac_Xflt_det1, frac_Yflt_det1, Xdth, Ydth;, skipline=7

   ; Sources on det.2 (lower), fits extension: [sci,1]
   adxy, dithered_head, RA_det2, Dec_det2, Xdth, Ydth	; Note! ADXY works in with IDL arrays starting at [0,0] !
   openw, 22, 'GC_xypos.tmp', width=300	; open tmp coord file
   for iaa=0,n_elements(Xdth)-1 do printf, 22, Xdth[iaa]+1., Ydth[iaa]+1. ;write x,y dith. coords in tmp file correcting to image coordinates (first pixel=[1,1])
   close, 22
   spawn, 'source get_dithered_coord.csh '+string(refimg)+' '+string(dithered_img)+$
    	' 1 GC_xypos.tmp backward'
   readcol, 'get_dithered_coord.log', frac_Xflt_det2, frac_Yflt_det2, Xdth, Ydth;, skipline=7
      
   frac_xcent=[frac_Xflt_det1,frac_Xflt_det2]   ; total X coord. array
   frac_ycent=[frac_Yflt_det1+(ydim/2.),frac_Yflt_det2]   ; total Y coord. array (det1 is converted to 4096 scale)
   forprint, frac_xcent, frac_ycent, textout=1	; for testing purposes
   
   xcent=fix(frac_xcent+0.5)   ;X,Y position are then rounded to integer pixels
   ycent=fix(frac_ycent+0.5)	 ; 
   
   xpixcent=xpixcent + ( long((frac_xcent*10.)+0.5)/10. - fix(frac_xcent+0.5) )   ;the residual shift is rounded to 1/10 of a pixel and later included in King
   ypixcent=ypixcent + ( long((frac_ycent*10.)+0.5)/10. - fix(frac_ycent+0.5) )   ; model as a center shift

endif




; -- Start cycling on the GC positions --
for j=0,numgc-1 do begin

    print, '-> Initiating source no.'+string(j+1)

    if (psfflag ne "psf") then begin
    	; --> When creating simulated GC uncomment one of the 2 the following 
    	;     blocks to choose between TinyTim or Anderson PSF!!

    	; Create the appropriate TyiniTim PSF
    	; Parameters are: detector, chip, xpos, ypos, filter, spectrum, spectrum file,
    	;		  psf size (arcsec), PSF file rootname, subsampling (y/n)   
    	;print, '  Creating appropriate TinyTim PSF...'
    	;psfroot='PSF_ellip_det'+strtrim(string(2-fix(float(ycent[j])/ydim*2.)),2)+'_'+strtrim(string(xcent[j]),2)+$
    	;	'_'+strtrim(string(ycent[j] mod 2048),2)+'_'+filt ;Define psf image root
    	;spawn, 'source makeHSTpsf.csh 15 '+strtrim(string(2-fix(ycent[j]/ydim*2.)),2)+' '+$
    	;	strtrim(string(xcent[j]),2)+' '+strtrim(string(ycent[j] mod 2048),2)+' '+$
    	;	filt+' 5 elliptical_spec.dat 3.0 '+psfroot
    	;psf_file=psfroot+'00.fits'	    ; Define file used for convolution
    	;spawn, 'rm -f '+psfroot+'.tt3' 	; remove useless TyniTim files
    	;spawn, 'rm -f '+psfroot+'00_psf.fits' ; remove useless intermediate TyniTim PSF files
		
    	; Read Anderson PSF - Note: relies on presence of Anderson PSF lib, special naming conventions and
    	;		   0.25 pixel sizes for resampling. Accurate for dithering 
    	print, '  Reading appropriate Anderson PSF...'
    	xlib=[0,512,1024,1536,2168,2800,3192,3584,4096] 	; PSF lib coordinates
    	ylib=[0,512,1024,1536,2048,2049,2560,3072,3584,4096]
    	xmin_dist=min( abs(xcent[j]-xlib), xmin_idx)		; find lib psf closest to actual position
    	ymin_dist=min( abs(ycent[j]-ylib), ymin_idx)
    	psf_file='anderson_psflib/psf606-flt-'+strtrim(xlib[xmin_idx],2)+$	; Define file used for convolution
    		    '-'+strtrim(ylib[ymin_idx],2)+'.fits'     

    
    	; Produce noisy GC model convolved with appropriate PSF
    
    	; Call procedure kingmod_addacs.pro
    	; - input param: output image name. Image dimensions and pix.scale (x y arcsec/pix),
    	;	    King parameters in image pixels (r0 c=r_t/r0), King model center in image pixels (x0 y0),
    	;	    King model total magnitude (in V band), observing time (sec), PSF image for convolution
 
    	imgname="king_"+string(r0[j],format='(f3.1)')+"_"+string(c[j],format='(i2)')+"_"+$
    		 string(totmag[j],format='(f4.1)')+"_psfconv.fits" ;Define output img name
 
    	print, '   Creating convolved GC...'
    	    kingmod_addacs, imgname, xdimgc, ydimgc, 0.049, r0[j], c[j], xpixcent[j], ypixcent[j], totmag[j], exptime, psf_file
    	    ;kingmod_addacs, imgname, xdimgc, ydimgc, 0.049, r0[j], c[j], xpixcent[j], ypixcent[j], totmag[j], exptime, 'no'	    ; to test without convolution

    	; Read the simulated GC image
        symGC_arr=readfits(imgname) 		    ; to add the convolved GC
        xdimgc2=xdimgc			    ; define new var to preserve the old one 
        ydimgc2=ydimgc			    ; in case you need to add only the PSF
    endif
    
    ; --> When simulating the PSF uncomment one of the 2 the following 
    ;	  blocks to choose between TinyTim or Anderson PSF!!
    if (psfflag eq "psf") then begin
    	; TyniTim PSF - Note: no frac.pixel shift implemented yet, not good for dithering
    	;psf_file=psfroot+'00.fits'				    ; Use TinyTim PSF
    	;symGC_arr=readfits(psf_file)	    ;read the PSF as if it is the GC file	
    	;xdimgc2=(size(symGC_arr))[1]			    ;derive x,y dim
    	;ydimgc2=(size(symGC_arr))[2]
    	;symGC_arr=symGC_arr[0:xdimgc2-2,0:ydimgc2-2]	    ;reduce file size by 1 to make it even as the GC file, and update size
    	;xdimgc2=(size(symGC_arr))[1]			    ; otherwise expression used for inclusion does not work
        ;ydimgc2=(size(symGC_arr))[2]			
        ;spawn, 'rm '+psfroot+'00.fits' ; remove TinyTim PSF files

    	; Anderson PSF - Note: relies on presence of Anderson PSF lib, special naming conventions and
    	;		       0.25 pixel sizes for resampling. Accurate for dithering 
    	xlib=[0,512,1024,1536,2168,2800,3192,3584,4096] 	; PSF lib coordinates
    	ylib=[0,512,1024,1536,2048,2049,2560,3072,3584,4096]
    	xmin_dist=min( abs(xcent[j]-xlib), xmin_idx)		; find lib psf closest to actual position
    	ymin_dist=min( abs(ycent[j]-ylib), ymin_idx)
    	psf_file='anderson_psflib/psf606-4xos-'+strtrim(xlib[xmin_idx],2)+'-'+strtrim(ylib[ymin_idx],2)+'.fits'     ; Use anderson PSF
    	PSF_osamp_arr=readfits(psf_file)    ;read the closest PSF as if it is the GC file    
    	symGC_arr=interpolate(PSF_osamp_arr, (indgen(26)-(xpixcent[j]-xdimgc/2.))*4,$	    ; resample Anderson PSF 4x4 to get FLT resolution
    	    (indgen(26)-(ypixcent[j]-ydimgc/2.))*4, /GRID, missing=0, cubic=-0.5 )
	;symGC_arr=symGC_arr/max(symGC_arr)*200.	; THIS LINE PRODUCES rescaled psfS, for test purposes only since this removes psf normalization
    	xdimgc2=(size(symGC_arr))[1]			    ;derive x,y dim
    	ydimgc2=(size(symGC_arr))[2]
    	
	; --- Normalize to total mag (change below if using different bands) ---
    	totflux = 10.^( -0.4 * ( totmag[j] + 21.12 ) )   ;Mag constant for V band

    	;;conv_fac = 5.07e18    ; cnts/s for HST ACS, F555W for const. flux source of 1 erg/s/cm^-2/Ang
    	conv_fac = 1.27e19     ; cnts/s for HST ACS, F606W for const. flux source of 1 erg/s/cm^-2/Ang
    	psf_norm = conv_fac * totflux / total( symGC_arr ) ; normalizing factor: total mag and convert in cnts/s
    	symGC_arr = symGC_arr*psf_norm*exptime	; normalize to total mag (comment if you want normalized psfs)
    endif
   
    ; Add simulated GC to ACS fov
    if ( (2-fix(float(ycent[j])/ydim*2.)) eq 1 ) then begin
    	print, 'X range of detector to add to:',xcent[j]-(xdimgc2/2), xcent[j]+(xdimgc2/2)-1	; debug: print adding range
    	print, 'Y range of detector to add to:',ycent[j]-(ydim/2)-(ydimgc2/2), ycent[j]-(ydim/2)+(ydimgc2/2)-1	; debug: print adding range
    	xini=max( [0,xcent[j]-(xdimgc2/2)] )	; initial image x-range to modify accounting for sources on edge
    	xfin=min( [4095,xcent[j]+(xdimgc2/2)-1] )  ; final image x-range to modify accounting for sources on edge
    	yini=max( [0,ycent[j]-(ydim/2)-(ydimgc2/2)] )	; initial image y-range to modify, accounting for sources on edge
    	yfin=min( [2047,ycent[j]-(ydim/2)+(ydimgc2/2)-1] )  ; final image y-range to modify, accounting for sources on edge
	print, 'X range accounting for sources on edge:', xini, xfin ; debug: adding range accounting for sources on edge
    	print, 'Y range accounting for sources on edge:', yini, yfin  ; debug: adding range accounting for sources on edge
    	xGCini=max( [0,(xdimgc2/2)-xcent[j]] )	; initial GC image x-range to modify, accounting for sources on edge
    	xGCfin=min( [(xdimgc2)-1,xdimgc2/2+(4095-xcent[j])] ) ; final GC image x-range to modify, accounting for sources on edge
    	yGCini=max( [0,(ydimgc2/2)-(ycent[j]-(ydim/2))] )	; initial GC image y-range to modify, accounting for sources on edge
    	yGCfin=min( [(ydimgc2)-1,ydimgc2/2+(4095-ycent[j])] ) ; final GC image y-range to modify, accounting for sources on edge
    	print, 'X GC range accounting for source on edge:', xGCini, xGCfin  ; debug: GC img adding range accounting for sources on edge
    	print, 'Y GC range accounting for source on edge:', yGCini, yGCfin  ; debug: GC img adding range accounting for sources on edge
    	acs_det1_img[ xini:xfin, yini:yfin ] = acs_det1_img[ xini:xfin, yini:yfin ] + symGC_arr[ xGCini:xGCfin, yGCini:yGCfin ]  ;det.1 (upper)
    endif
    if ( (2-fix(float(ycent[j])/ydim*2.)) eq 2 ) then begin
    	print, 'X range of detector to add to:',xcent[j]-(xdimgc2/2), xcent[j]+(xdimgc2/2)-1	; debug: print adding range
    	print, 'Y range of detector to add to:',ycent[j]-(ydimgc2/2), ycent[j]+(ydimgc2/2)-1	; debug: print adding range
    	xini=max( [0,xcent[j]-(xdimgc2/2)] )	; initial image x-range to modify accounting for sources on edge
    	xfin=min( [4095,xcent[j]+(xdimgc2/2)-1] )  ; final image x-range to modify accounting for sources on edge
    	yini=max( [0,ycent[j]-(ydimgc2/2)] )	; initial image y-range to modify, accounting for sources on edge
    	yfin=min( [2047,ycent[j]+(ydimgc2/2)-1] )  ; final image y-range to modify, accounting for sources on edge
	print, 'X range accounting for sources on edge:', xini, xfin ; debug: adding range accounting for sources on edge
    	print, 'Y range accounting for sources on edge:', yini, yfin  ; debug: adding range accounting for sources on edge
    	xGCini=max( [0,(xdimgc2/2)-xcent[j]] )	; initial GC image x-range to modify, accounting for sources on edge
    	xGCfin=min( [(xdimgc2)-1,xdimgc2/2+(4095-xcent[j])] ) ; final GC image x-range to modify, accounting for sources on edge
    	yGCini=max( [0,(ydimgc2/2)-ycent[j]] )	; initial GC image y-range to modify, accounting for sources on edge
    	yGCfin=min( [(ydimgc2)-1,ydimgc2/2+(2047-ycent[j])] ) ; final GC image y-range to modify, accounting for sources on edge
    	print, 'X GC range accounting for source on edge:', xGCini, xGCfin  ; debug: GC img adding range accounting for sources on edge
    	print, 'Y GC range accounting for source on edge:', yGCini, yGCfin  ; debug: GC img adding range accounting for sources on edge
     	acs_det2_img[ xini:xfin, yini:yfin ] = acs_det2_img[ xini:xfin, yini:yfin ] + symGC_arr[ xGCini:xGCfin, yGCini:yGCfin ]  ;det.1 (upper)
   endif
    
    ; Write simulated GC parameters in file
    printf, 11, j+1, string(RA[j],format='(d15.8)'), string(Dec[j],format='(d15.8)'), $
    	    	 xcent[j], ycent[j], r0[j], c[j], totmag[j], xpixcent[j], ypixcent[j], (2-fix(ycent[j]/ydim*2.)), "     "+psf_file
   
endfor

; ---- Output image -----
if (finimgflag eq "a") then begin

  ; ---- Add GC to original image ------
  print, '   Adding GC to original image '+string(refimg)+' creating new image '+$
    string(outimg)

  fits_open, refimg, fcb		; open old file for reading  
  fits_open, outimg, fcb2, /write	; open new file for writing

  for i = 1, fcb.nextend do begin             ; i=0 reads PDU
      fits_read, fcb, det_ext, det_head  ; read each fits extension  
      ; update 'sci' extensions adding simulated GC to ACS image
      if (fcb.extname[i] eq 'SCI' and fcb.extver[i] eq 1) then det_ext=det_ext+acs_det2_img	; det. 2 (lower, fits extension [sci,1])
      if (fcb.extname[i] eq 'ERR' and fcb.extver[i] eq 1) then det_ext=sqrt(det_ext^2+acs_det2_img)	; add errors to det.2
      if (fcb.extname[i] eq 'SCI' and fcb.extver[i] eq 2) then det_ext=det_ext+acs_det1_img	; det. 1 (upper, fits extension [sci,2])
      if (fcb.extname[i] eq 'ERR' and fcb.extver[i] eq 2) then det_ext=sqrt(det_ext^2+acs_det1_img)	; add errors to det.1
      fits_write, fcb2, det_ext, det_head
  end

  fits_close, fcb
  fits_close, fcb2

 if (weightflag eq 'y') then begin	; add Poisson noise to mask if requested
   ; Mask 1 (det. 2 (lower, corresponding to fits extension [err,1])) 
   fits_read, simmask1, mask_img, mask_head  ; read mask image
   help, mask_img
   mask_flag=mask_img	; make 0,1 flag mask to preserve null pixels in masks
   mask_flag(where(mask_img ne 0.))=1.
   mask_img=1./((1./mask_img)+float(acs_det2_img))*mask_flag   ; add Poisson noise to mask leaving 0's unchanges
   help, acs_det2_img, mask_img
   fits_write, simmask1, mask_img, mask_head
   ; Mask 2 (det. 1 (upper, corresponding to fits extension [err,2])) 
   fits_read, simmask2, mask_img, mask_head  ; read mask extension
   mask_flag=mask_img	; make 0,1 flag mask to preserve null pixels in masks
   mask_flag(where(mask_img ne 0.))=1.
   mask_img=1./((1./mask_img)+float(acs_det1_img))*mask_flag   ; add Poisson noise to mask leaving 0's unchanges
   fits_write, simmask2, mask_img, mask_head
 endif
  
 
  
endif else begin

  ; ---- Create new simulated ACS image ------
  print, '   Writing final simulated image '+string(outimg)

  fits_open, refimg, fcb		; open old file for reading  
  fits_open, outimg, fcb2, /write	; open new file for writing

  for i = 1, fcb.nextend do begin             ; i=0 reads PDU
      fits_read, fcb, det_ext, det_head  ; read each fits extension  
      ; update 'sci' extensions replacing with simulated GC to ACS image
      if (fcb.extname[i] eq 'SCI' and fcb.extver[i] eq 1) then det_ext=acs_det2_img	; det. 2 (lower, fits extension [sci,1])
      ;if (fcb.extname[i] eq 'ERR' and fcb.extver[i] eq 1) then det_ext=sqrt(acs_det2_img)	; make poisson error extensio for det. 2 !WARNING: inaccurate, better leave commented and have empty error extension
      if (fcb.extname[i] eq 'SCI' and fcb.extver[i] eq 2) then det_ext=acs_det1_img	; det. 1 (upper, fits extension [sci,2])
      ;if (fcb.extname[i] eq 'ERR' and fcb.extver[i] eq 2) then det_ext=sqrt(acs_det1_img)	; make poisson error extensio for det. 1 !WARNING: inaccurate, better leave commented and have empty error extension
      fits_write, fcb2, det_ext, det_head
  end

  fits_close, fcb
  fits_close, fcb2

endelse

spawn, 'set chck = `ls | grep _psfconv.fits` ; if ( $#chck > 0 ) rm -f king_*_psfconv.fits'	; remove PSF-convolved GC fits images


close, /all
end
